@@ -1,4 +1,4 @@
-from datetime import datetime
+from datetime import date, datetime
 from typing import List, Union
 
 from pydantic import AnyUrl, Field, root_validator, validator
@@ -12,6 +12,7 @@ from hsmodels.schemas.fields import (
     CellInformation,
     FieldInformation,
     GeometryInformation,
+    ModelProgramFile,
     MultidimensionalBoxSpatialReference,
     MultidimensionalPointSpatialReference,
     PeriodCoverage,
@@ -21,13 +22,19 @@ from hsmodels.schemas.fields import (
     TimeSeriesResult,
     Variable,
 )
-from hsmodels.schemas.rdf.validators import language_constraint
-from hsmodels.schemas.root_validators import parse_abstract, parse_additional_metadata, parse_url, split_coverages
+from hsmodels.schemas.rdf.validators import language_constraint, subjects_constraint
+from hsmodels.schemas.root_validators import (
+    normalize_additional_metadata,
+    parse_abstract,
+    parse_additional_metadata,
+    parse_file_types,
+    parse_url,
+    split_coverages,
+)
 from hsmodels.schemas.validators import (
     parse_multidimensional_spatial_reference,
     parse_spatial_coverage,
     parse_spatial_reference,
-    normalize_additional_metadata,
 )
 
 
@@ -74,9 +81,10 @@ class BaseAggregationMetadata(BaseMetadata):
     _parse_coverages = root_validator(pre=True, allow_reuse=True)(split_coverages)
     _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
 
+    _subjects_constraint = validator('subjects', allow_reuse=True)(subjects_constraint)
     _language_constraint = validator('language', allow_reuse=True)(language_constraint)
     _parse_spatial_coverage = validator("spatial_coverage", allow_reuse=True, pre=True)(parse_spatial_coverage)
-    _normalize_additional_metadata = validator("additional_metadata", allow_reuse=True, pre=True)(normalize_additional_metadata)
+    _normalize_additional_metadata = root_validator(allow_reuse=True, pre=True)(normalize_additional_metadata)
 
 
 class GeographicRasterMetadata(BaseAggregationMetadata):
@@ -91,8 +99,7 @@ class GeographicRasterMetadata(BaseAggregationMetadata):
     class Config:
         title = 'Geographic Raster Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -130,8 +137,7 @@ class GeographicFeatureMetadata(BaseAggregationMetadata):
     class Config:
         title = 'Geographic Feature Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -170,8 +176,7 @@ class MultidimensionalMetadata(BaseAggregationMetadata):
     class Config:
         title = 'Multidimensional Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -208,8 +213,7 @@ class ReferencedTimeSeriesMetadata(BaseAggregationMetadata):
     class Config:
         title = 'Referenced Time Series Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -232,8 +236,7 @@ class FileSetMetadata(BaseAggregationMetadata):
     class Config:
         title = 'File Set Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -255,8 +258,7 @@ class SingleFileMetadata(BaseAggregationMetadata):
     class Config:
         title = 'Single File Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -281,8 +283,7 @@ class TimeSeriesMetadata(BaseAggregationMetadata):
     class Config:
         title = 'Time Series Aggregation Metadata'
 
-        schema_config = {'read_only': ['type', 'url'],
-                         'dictionary_field': ['additional_metadata']}
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
     type: AggregationType = Field(
         const=True,
@@ -300,3 +301,117 @@ class TimeSeriesMetadata(BaseAggregationMetadata):
     abstract: str = Field(default=None, title="Abstract", description="A string containing a summary of a aggregation")
 
     _parse_abstract = root_validator(pre=True, allow_reuse=True)(parse_abstract)
+
+
+class ModelProgramMetadata(BaseAggregationMetadata):
+    """
+    A class used to represent the metadata associated with a model program aggregation
+    """
+
+    class Config:
+        title = 'Model Program Aggregation Metadata'
+
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.ModelProgramAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    version: str = Field(
+        default=None, title="Version", description="The software version or build number of the model", max_length=255
+    )
+
+    name: str = Field(
+        default="Unknown Model Program",
+        max_length=255,
+        title="Model Program Name",
+        description="A string expressing the name of the model program",
+    )
+
+    programming_languages: List[str] = Field(
+        default=[],
+        max_length=100,
+        title="Programming Languages",
+        description="The programming languages that the model is written in",
+    )
+
+    operating_systems: List[str] = Field(
+        default=[],
+        max_length=100,
+        title="Operating Systems",
+        description="Compatible operating systems to setup and run the model",
+    )
+
+    release_date: date = Field(
+        default=None, title="Release Date", description="The date that this version of the model was released"
+    )
+
+    website: AnyUrl = Field(
+        default=None,
+        title='Website',
+        description='A URL to a website describing the model that is maintained by the model developers',
+    )
+
+    code_repository: AnyUrl = Field(
+        default=None,
+        title='Software Repository',
+        description='A URL to the source code repository for the model code (e.g., git, mercurial, svn, etc.)',
+    )
+
+    file_types: List[ModelProgramFile] = Field(
+        default=None, title='File Types', description='File types used by the model program'
+    )
+
+    program_schema_json: AnyUrl = Field(
+        default=None,
+        title='Model program schema',
+        description='A url to the JSON metadata schema for the model program',
+    )
+
+    _parse_file_types = root_validator(pre=True, allow_reuse=True)(parse_file_types)
+
+
+class ModelInstanceMetadata(BaseAggregationMetadata):
+    """
+    A class used to represent the metadata associated with a model instance aggregation
+    """
+
+    class Config:
+        title = 'Model Instance Aggregation Metadata'
+
+        schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.ModelInstanceAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    includes_model_output: bool = Field(
+        title="Includes Model Output",
+        description="Indicates whether model output files are included in the aggregation",
+    )
+
+    executed_by: AnyUrl = Field(
+        default=None,
+        title="Executed By",
+        description="A URL to the Model Program that can be used to execute this model instance",
+    )
+
+    program_schema_json: AnyUrl = Field(
+        default=None,
+        title="JSON Metadata schema URL",
+        description="A URL to the JSON metadata schema for the related model program",
+    )
+
+    program_schema_json_values: AnyUrl = Field(
+        default=None,
+        title="JSON metadata schema values URL",
+        description="A URL to a JSON file containing the metadata values conforming to the JSON metadata schema for the related model program",
+    )
